@@ -50,7 +50,7 @@
 #include <panel/panel-plugin-external.h>
 
 #define AUTOSAVE_INTERVAL (10 * 60)
-#define MIGRATE_BIN       LIBEXECDIR G_DIR_SEPARATOR_S "panel" G_DIR_SEPARATOR_S "migrate"
+#define MIGRATE_BIN       LIBDIR G_DIR_SEPARATOR_S "panel" G_DIR_SEPARATOR_S "migrate"
 
 
 
@@ -567,7 +567,7 @@ panel_application_plugin_provider_signal (XfcePanelPluginProvider       *provide
 
     case PROVIDER_SIGNAL_FOCUS_PLUGIN:
        /* focus the panel window (as part of focusing a widget within the plugin) */
-       gtk_window_present_with_time (GTK_WINDOW (window), GDK_CURRENT_TIME);
+       panel_window_focus (window);
        break;
 
     case PROVIDER_SIGNAL_SHOW_CONFIGURE:
@@ -1413,17 +1413,30 @@ panel_application_logout (void)
 {
   XfceSMClient *sm_client;
   GError       *error = NULL;
+  const gchar  *command = "xfce4-session-logout";
 
   /* first try to session client to logout else fallback and spawn xfce4-session-logout */
   sm_client = xfce_sm_client_get ();
   if (xfce_sm_client_is_connected (sm_client))
     {
       xfce_sm_client_request_shutdown (sm_client, XFCE_SM_CLIENT_SHUTDOWN_HINT_ASK);
+
+      return;
     }
-  else if (!g_spawn_command_line_async ("xfce4-session-logout", &error))
+  else if (g_getenv ("SESSION_MANAGER") == NULL)
+    {
+      if (xfce_dialog_confirm (NULL, GTK_STOCK_QUIT, NULL,
+          _("If you have started Xfce without session manager, this will close the X server."),
+          _("Are you sure you want to quit the panel?")))
+        command = "xfce4-panel --quit";
+      else
+        return;
+    }
+
+  if (!g_spawn_command_line_async (command, &error))
     {
       xfce_dialog_show_error (NULL, error, _("Failed to execute command \"%s\""),
-                              "xfce4-session-logout");
+                              command);
       g_error_free (error);
     }
 }
