@@ -27,6 +27,11 @@
 #include <math.h>
 #endif
 
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#include <X11/Xlib.h>
+#endif
+
 #include <exo/exo.h>
 #include <common/panel-private.h>
 #include <common/panel-debug.h>
@@ -95,7 +100,7 @@ static gboolean     panel_window_button_press_event         (GtkWidget        *w
 static gboolean     panel_window_button_release_event       (GtkWidget        *widget,
                                                              GdkEventButton   *event);
 static void         panel_window_grab_notify                (GtkWidget        *widget,
-                                                             gboolean         was_grabbed);
+                                                             gboolean          was_grabbed);
 static void         panel_window_size_request               (GtkWidget        *widget,
                                                              GtkRequisition   *requisition);
 static void         panel_window_size_allocate              (GtkWidget        *widget,
@@ -264,6 +269,7 @@ struct _PanelWindow
   AutohideState        autohide_state;
   guint                autohide_timeout_id;
   gint                 autohide_block;
+  gint                 autohide_grab_block;
 
   /* popup/down delay from gtk style */
   gint                 popup_delay;
@@ -430,6 +436,7 @@ panel_window_init (PanelWindow *window)
   window->autohide_state = AUTOHIDE_DISABLED;
   window->autohide_timeout_id = 0;
   window->autohide_block = 0;
+  window->autohide_grab_block = 0;
   window->popup_delay = DEFAULT_POPUP_DELAY;
   window->popdown_delay = DEFAULT_POPDOWN_DELAY;
   window->base_x = -1;
@@ -1005,14 +1012,30 @@ panel_window_grab_notify (GtkWidget *widget,
                           gboolean   was_grabbed)
 {
   PanelWindow *window = PANEL_WINDOW (widget);
+  GtkWidget   *current;
 
-  /* avoid hiding the panel when the window is grabbed. this
-   * (for example) happens when the user drags in the pager plugin
-   * see bug #4597 */
-  if (was_grabbed)
-    panel_window_thaw_autohide (window);
-  else
-    panel_window_freeze_autohide (window);
+  current = gtk_grab_get_current ();
+  if (GTK_IS_MENU_SHELL (current))
+    {
+      /* don't act on menu grabs, they should be registered through the
+       * plugin if they should block autohide */
+      window->autohide_grab_block++;
+    }
+  else if (window->autohide_grab_block > 0)
+    {
+      /* drop previous menu block */
+      window->autohide_grab_block--;
+    }
+  else if (window->autohide_grab_block == 0)
+    {
+      /* avoid hiding the panel when the window is grabbed. this
+       * (for example) happens when the user drags in the pager plugin
+       * see bug #4597 */
+      if (was_grabbed)
+        panel_window_thaw_autohide (window);
+      else
+        panel_window_freeze_autohide (window);
+    }
 }
 
 
@@ -2416,3 +2439,38 @@ panel_window_get_locked (PanelWindow *window)
 
   return window->locked;
 }
+
+
+
+void
+panel_window_focus (PanelWindow *window)
+{
+#ifdef GDK_WINDOWING_X11
+  XClientMessageEvent event;
+
+  panel_return_if_fail (PANEL_IS_WINDOW (window));
+  panel_return_if_fail (GTK_WIDGET_REALIZED (window));
+
+  /* we need a slightly custom version of the call through Gtk+ to
+   * properly focus the panel when a plugin calls
+   * xfce_panel_plugin_focus_widget() */
+  event.type = ClientMessage;
+  event.window = GDK_WINDOW_XID (GTK_WIDGET (window)->window);
+  event.message_type = gdk_x11_get_xatom_by_name ("_NET_ACTIVE_WINDOW");
+  event.format = 32;
+  event.data.l[0] = 0;
+
+  gdk_error_trap_push ();
+
+  XSendEvent (GDK_DISPLAY (), GDK_ROOT_WINDOW (), False,
+              StructureNotifyMask, (XEvent *) &event);
+
+  gdk_flush ();
+
+  if (gdk_error_trap_pop () != 0)
+    g_critical ("Failed to focus panel window");
+#else
+  /* our best guess on non-x11 clients */
+  gtk_window_present (GTK_WINDOW (window));
+#endif
+}
